@@ -3,9 +3,10 @@ from copy import copy
 from datetime import datetime, timezone
 from redis.asyncio.client import Redis
 from typing import Dict, List, Optional
-from uuid import uuid4
+from uuid import UUID
 from maleo_soma.dtos.configurations.cache.redis import RedisCacheNamespaces
 from maleo_soma.enums.environment import Environment
+from maleo_soma.enums.expiration import Expiration
 from maleo_soma.enums.logging import LogLevel
 from maleo_soma.enums.operation import OperationTarget
 from maleo_soma.exceptions import NotImplemented as MaleoNotImplemented
@@ -29,25 +30,29 @@ from maleo_soma.schemas.operation.resource.result import (
     ReadMultipleResourceOperationResult,
 )
 from maleo_soma.schemas.operation.timestamp import OperationTimestamp
+from maleo_soma.schemas.pagination import StrictPagination
 from maleo_soma.schemas.request import RequestContext
 from maleo_soma.schemas.response import (
     SingleDataResponseSchema,
     MultipleDataResponseSchema,
 )
 from maleo_soma.schemas.service import ServiceContext
-from maleo_soma.types.base import OptionalUUID
 from maleo_soma.utils.cache import build_key
 from maleo_soma.utils.logging import ClientLogger
 from maleo_metadata.client.controllers import MaleoMetadataMedicalRoleControllers
 from maleo_metadata.enums.general import ClientControllerType
 from maleo_metadata.constants.medical_role import RESOURCE
 from maleo_metadata.schemas.data.medical_role import MedicalRoleDataSchema
-from maleo_metadata.schemas.parameter.client.medical_role import ReadMultipleParameter
+from maleo_metadata.schemas.parameter.client.medical_role import (
+    ReadMultipleParameter,
+    ReadMultipleRootSpecializationsParameter,
+    ReadMultipleSpecializationsParameter,
+)
 from maleo_metadata.schemas.parameter.general.medical_role import ReadSingleParameter
 
 
 class MaleoMetadataMedicalRoleClientService(MaleoClientService):
-    RESOURCE = RESOURCE
+    _resource = RESOURCE
 
     def __init__(
         self,
@@ -72,7 +77,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
         self._controllers = controllers
         self._namespace = self._redis_namespaces.create(
             self._key,
-            "medical_role",
+            self._resource.aggregate(),
             origin=self._CACHE_ORIGIN,
             layer=self._CACHE_LAYER,
         )
@@ -86,16 +91,17 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
 
     async def read_medical_roles(
         self,
+        operation_id: UUID,
         request_context: RequestContext,
         authentication: Authentication,
         parameters: ReadMultipleParameter,
         controller_type: ClientControllerType = ClientControllerType.HTTP,
-        operation_id: OptionalUUID = None,
         authorization: Optional[Authorization] = None,
         headers: Optional[Dict[str, str]] = None,
-    ) -> ReadMultipleResourceOperationResult[MedicalRoleDataSchema, None]:
+    ) -> ReadMultipleResourceOperationResult[
+        MedicalRoleDataSchema, StrictPagination, None
+    ]:
         """Retrieve medical roles from MaleoMetadata"""
-        operation_id = operation_id if operation_id is not None else uuid4()
         operation_context = copy(self._default_operation_context)
         operation_action = ReadResourceOperationAction()
         executed_at = datetime.now(tz=timezone.utc)
@@ -130,18 +136,20 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
         # Define full function string
         function = f"{qualname}({json.dumps(positional_arguments)}|{json.dumps(keyword_arguments)})"
 
-        # Define full cache key
-        key = build_key(module, function, namespace=self._namespace)
+        # Define full cache cache_key
+        cache_key = build_key(module, function, namespace=self._namespace)
 
         # Check redis for data
-        result_str = await self._redis.get(key)
+        result_str = await self._redis.get(cache_key)
 
         if result_str is not None:
             completed_at = datetime.now(tz=timezone.utc)
             result = ReadMultipleResourceOperationResult[
-                MedicalRoleDataSchema, None
+                MedicalRoleDataSchema, StrictPagination, None
             ].model_validate(json.loads(result_str))
-            ReadMultipleResourceOperationSchema[MedicalRoleDataSchema, None](
+            ReadMultipleResourceOperationSchema[
+                MedicalRoleDataSchema, StrictPagination, None
+            ](
                 service_context=self._service_context,
                 id=operation_id,
                 context=operation_context,
@@ -154,9 +162,11 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 request_context=request_context,
                 authentication=authentication,
                 action=operation_action,
-                resource=self.RESOURCE,
+                resource=self._resource,
                 result=result,
-            ).log(self._logger, level=LogLevel.INFO)
+            ).log(
+                self._logger, level=LogLevel.INFO
+            )
             return result
 
         operation_context.target.type = OperationTarget.CONTROLLER
@@ -164,6 +174,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
         # Retrieve medical roles using chosen controller
         if controller_type is ClientControllerType.HTTP:
             response = await self._controllers.http.read_medical_roles(
+                operation_id=operation_id,
                 authentication=authentication,
                 parameters=parameters,
                 authorization=authorization,
@@ -172,21 +183,23 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
             if response.is_success:
                 completed_at = datetime.now(tz=timezone.utc)
                 validated_response = MultipleDataResponseSchema[
-                    MedicalRoleDataSchema, None
+                    MedicalRoleDataSchema, StrictPagination, None
                 ].model_validate(response.json())
                 data = DataPair[List[MedicalRoleDataSchema], None](
                     old=validated_response.data,
                     new=None,
                 )
                 result = ReadMultipleResourceOperationResult[
-                    MedicalRoleDataSchema, None
+                    MedicalRoleDataSchema, StrictPagination, None
                 ](
                     data=data,
                     pagination=validated_response.pagination,
                     metadata=None,
                     other=None,
                 )
-                ReadMultipleResourceOperationSchema[MedicalRoleDataSchema, None](
+                ReadMultipleResourceOperationSchema[
+                    MedicalRoleDataSchema, StrictPagination, None
+                ](
                     service_context=self._service_context,
                     id=operation_id,
                     context=operation_context,
@@ -199,9 +212,14 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                     request_context=request_context,
                     authentication=authentication,
                     action=operation_action,
-                    resource=self.RESOURCE,
+                    resource=self._resource,
                     result=result,
-                ).log(self._logger, level=LogLevel.INFO)
+                ).log(
+                    self._logger, level=LogLevel.INFO
+                )
+                await self._redis.set(
+                    cache_key, result.model_dump_json(), Expiration.EXP_1WK
+                )
                 return result
 
             self._raise_resource_http_request_error(
@@ -212,7 +230,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 operation_action=operation_action,
                 request_context=request_context,
                 authentication=authentication,
-                resource=self.RESOURCE,
+                resource=self._resource,
             )
         else:
             completed_at = datetime.now(tz=timezone.utc)
@@ -229,23 +247,24 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 operation_action=operation_action,
                 request_context=request_context,
                 authentication=authentication,
-                resource=self.RESOURCE,
+                resource=self._resource,
             )
             error.operation_schema.log(self._logger, LogLevel.CRITICAL)
             raise error
 
-    async def read_medical_role_specializations(
+    async def read_medical_roles_specializations(
         self,
+        operation_id: UUID,
         request_context: RequestContext,
         authentication: Authentication,
-        parameters: ReadMultipleParameter,
+        parameters: ReadMultipleRootSpecializationsParameter,
         controller_type: ClientControllerType = ClientControllerType.HTTP,
-        operation_id: OptionalUUID = None,
         authorization: Optional[Authorization] = None,
         headers: Optional[Dict[str, str]] = None,
-    ) -> ReadMultipleResourceOperationResult[MedicalRoleDataSchema, None]:
+    ) -> ReadMultipleResourceOperationResult[
+        MedicalRoleDataSchema, StrictPagination, None
+    ]:
         """Retrieve medical roles from MaleoMetadata"""
-        operation_id = operation_id if operation_id is not None else uuid4()
         operation_context = copy(self._default_operation_context)
         operation_action = ReadResourceOperationAction()
         executed_at = datetime.now(tz=timezone.utc)
@@ -280,18 +299,20 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
         # Define full function string
         function = f"{qualname}({json.dumps(positional_arguments)}|{json.dumps(keyword_arguments)})"
 
-        # Define full cache key
-        key = build_key(module, function, namespace=self._namespace)
+        # Define full cache cache_key
+        cache_key = build_key(module, function, namespace=self._namespace)
 
         # Check redis for data
-        result_str = await self._redis.get(key)
+        result_str = await self._redis.get(cache_key)
 
         if result_str is not None:
             completed_at = datetime.now(tz=timezone.utc)
             result = ReadMultipleResourceOperationResult[
-                MedicalRoleDataSchema, None
+                MedicalRoleDataSchema, StrictPagination, None
             ].model_validate(json.loads(result_str))
-            ReadMultipleResourceOperationSchema[MedicalRoleDataSchema, None](
+            ReadMultipleResourceOperationSchema[
+                MedicalRoleDataSchema, StrictPagination, None
+            ](
                 service_context=self._service_context,
                 id=operation_id,
                 context=operation_context,
@@ -300,20 +321,23 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                     completed_at=completed_at,
                     duration=(completed_at - executed_at).total_seconds(),
                 ),
-                summary="Successfully retrieved multiple medical roles",
+                summary="Successfully retrieved multiple root medical roles specializations",
                 request_context=request_context,
                 authentication=authentication,
                 action=operation_action,
-                resource=self.RESOURCE,
+                resource=self._resource,
                 result=result,
-            ).log(self._logger, level=LogLevel.INFO)
+            ).log(
+                self._logger, level=LogLevel.INFO
+            )
             return result
 
         operation_context.target.type = OperationTarget.CONTROLLER
 
         # Retrieve medical roles using chosen controller
         if controller_type is ClientControllerType.HTTP:
-            response = await self._controllers.http.read_medical_roles(
+            response = await self._controllers.http.read_medical_roles_specializations(
+                operation_id=operation_id,
                 authentication=authentication,
                 parameters=parameters,
                 authorization=authorization,
@@ -322,21 +346,23 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
             if response.is_success:
                 completed_at = datetime.now(tz=timezone.utc)
                 validated_response = MultipleDataResponseSchema[
-                    MedicalRoleDataSchema, None
+                    MedicalRoleDataSchema, StrictPagination, None
                 ].model_validate(response.json())
                 data = DataPair[List[MedicalRoleDataSchema], None](
                     old=validated_response.data,
                     new=None,
                 )
                 result = ReadMultipleResourceOperationResult[
-                    MedicalRoleDataSchema, None
+                    MedicalRoleDataSchema, StrictPagination, None
                 ](
                     data=data,
                     pagination=validated_response.pagination,
                     metadata=None,
                     other=None,
                 )
-                ReadMultipleResourceOperationSchema[MedicalRoleDataSchema, None](
+                ReadMultipleResourceOperationSchema[
+                    MedicalRoleDataSchema, StrictPagination, None
+                ](
                     service_context=self._service_context,
                     id=operation_id,
                     context=operation_context,
@@ -345,13 +371,18 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                         completed_at=completed_at,
                         duration=(completed_at - executed_at).total_seconds(),
                     ),
-                    summary="Successfully retrieved multiple medical roles",
+                    summary="Successfully retrieved multiple root medical roles specializations",
                     request_context=request_context,
                     authentication=authentication,
                     action=operation_action,
-                    resource=self.RESOURCE,
+                    resource=self._resource,
                     result=result,
-                ).log(self._logger, level=LogLevel.INFO)
+                ).log(
+                    self._logger, level=LogLevel.INFO
+                )
+                await self._redis.set(
+                    cache_key, result.model_dump_json(), Expiration.EXP_1WK
+                )
                 return result
 
             self._raise_resource_http_request_error(
@@ -362,7 +393,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 operation_action=operation_action,
                 request_context=request_context,
                 authentication=authentication,
-                resource=self.RESOURCE,
+                resource=self._resource,
             )
         else:
             completed_at = datetime.now(tz=timezone.utc)
@@ -379,24 +410,23 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 operation_action=operation_action,
                 request_context=request_context,
                 authentication=authentication,
-                resource=self.RESOURCE,
+                resource=self._resource,
             )
             error.operation_schema.log(self._logger, LogLevel.CRITICAL)
             raise error
 
     async def read_medical_role(
         self,
+        operation_id: UUID,
         request_context: RequestContext,
         authentication: Authentication,
         parameters: ReadSingleParameter,
         controller_type: ClientControllerType = ClientControllerType.HTTP,
-        operation_id: OptionalUUID = None,
         authorization: Optional[Authorization] = None,
         headers: Optional[Dict[str, str]] = None,
     ) -> ReadSingleResourceOperationResult[MedicalRoleDataSchema, None]:
         """Retrieve medical role from MaleoMetadata"""
 
-        operation_id = operation_id if operation_id is not None else uuid4()
         operation_context = copy(self._default_operation_context)
         operation_action = ReadResourceOperationAction()
         executed_at = datetime.now(tz=timezone.utc)
@@ -431,11 +461,11 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
         # Define full function string
         function = f"{qualname}({json.dumps(positional_arguments)}|{json.dumps(keyword_arguments)})"
 
-        # Define full cache key
-        key = build_key(module, function, namespace=self._namespace)
+        # Define full cache cache_key
+        cache_key = build_key(module, function, namespace=self._namespace)
 
         # Check redis for data
-        result_str = await self._redis.get(key)
+        result_str = await self._redis.get(cache_key)
 
         if result_str is not None:
             completed_at = datetime.now(tz=timezone.utc)
@@ -455,7 +485,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 request_context=request_context,
                 authentication=authentication,
                 action=operation_action,
-                resource=self.RESOURCE,
+                resource=self._resource,
                 result=result,
             ).log(self._logger, level=LogLevel.INFO)
             return result
@@ -465,6 +495,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
         # Retrieve medical roles using chosen controller
         if controller_type is ClientControllerType.HTTP:
             response = await self._controllers.http.read_medical_role(
+                operation_id=operation_id,
                 authentication=authentication,
                 parameters=parameters,
                 authorization=authorization,
@@ -498,9 +529,12 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                     request_context=request_context,
                     authentication=authentication,
                     action=operation_action,
-                    resource=self.RESOURCE,
+                    resource=self._resource,
                     result=result,
                 ).log(self._logger, level=LogLevel.INFO)
+                await self._redis.set(
+                    cache_key, result.model_dump_json(), Expiration.EXP_1WK
+                )
                 return result
 
             self._raise_resource_http_request_error(
@@ -511,7 +545,7 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 operation_action=operation_action,
                 request_context=request_context,
                 authentication=authentication,
-                resource=self.RESOURCE,
+                resource=self._resource,
             )
         else:
             completed_at = datetime.now(tz=timezone.utc)
@@ -527,8 +561,171 @@ class MaleoMetadataMedicalRoleClientService(MaleoClientService):
                 operation_summary=f"The provided controller type '{controller_type}' did not exists",
                 operation_action=operation_action,
                 request_context=request_context,
-                resource=self.RESOURCE,
+                resource=self._resource,
                 authentication=authentication,
             )
             error.operation_schema.log(self._logger, LogLevel.CRITICAL)
             raise error
+
+    async def read_medical_role_specializations(
+        self,
+        operation_id: UUID,
+        request_context: RequestContext,
+        authentication: Authentication,
+        parameters: ReadMultipleSpecializationsParameter,
+        controller_type: ClientControllerType = ClientControllerType.HTTP,
+        authorization: Optional[Authorization] = None,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> ReadMultipleResourceOperationResult[
+        MedicalRoleDataSchema, StrictPagination, None
+    ]:
+        """Retrieve medical roles from MaleoMetadata"""
+        operation_context = copy(self._default_operation_context)
+        operation_action = ReadResourceOperationAction()
+        executed_at = datetime.now(tz=timezone.utc)
+
+        operation_context.target.type = OperationTarget.CACHE
+
+        # Get function identifier
+        func = self.__class__
+        module, qualname = func.__module__, func.__qualname__
+
+        # Define arguments being used in this function
+        positional_arguments = []
+        keyword_arguments = {
+            "authentication": authentication.model_dump(
+                mode="json",
+                exclude={
+                    "credentials": {
+                        "token": {
+                            "payload": {
+                                "iat_dt",
+                                "iat",
+                                "exp_dt",
+                                "exp",
+                            }
+                        }
+                    }
+                },
+            ),
+            "parameters": parameters.model_dump(mode="json"),
+        }
+
+        # Define full function string
+        function = f"{qualname}({json.dumps(positional_arguments)}|{json.dumps(keyword_arguments)})"
+
+        # Define full cache cache_key
+        cache_key = build_key(module, function, namespace=self._namespace)
+
+        # Check redis for data
+        result_str = await self._redis.get(cache_key)
+
+        if result_str is not None:
+            completed_at = datetime.now(tz=timezone.utc)
+            result = ReadMultipleResourceOperationResult[
+                MedicalRoleDataSchema, StrictPagination, None
+            ].model_validate(json.loads(result_str))
+            ReadMultipleResourceOperationSchema[
+                MedicalRoleDataSchema, StrictPagination, None
+            ](
+                service_context=self._service_context,
+                id=operation_id,
+                context=operation_context,
+                timestamp=OperationTimestamp(
+                    executed_at=executed_at,
+                    completed_at=completed_at,
+                    duration=(completed_at - executed_at).total_seconds(),
+                ),
+                summary="Successfully retrieved multiple medical role specializations",
+                request_context=request_context,
+                authentication=authentication,
+                action=operation_action,
+                resource=self._resource,
+                result=result,
+            ).log(
+                self._logger, level=LogLevel.INFO
+            )
+            return result
+
+        operation_context.target.type = OperationTarget.CONTROLLER
+
+        # Retrieve medical roles using chosen controller
+        if controller_type is ClientControllerType.HTTP:
+            response = await self._controllers.http.read_medical_role_specializations(
+                operation_id=operation_id,
+                authentication=authentication,
+                parameters=parameters,
+                authorization=authorization,
+                headers=headers,
+            )
+            if response.is_success:
+                completed_at = datetime.now(tz=timezone.utc)
+                validated_response = MultipleDataResponseSchema[
+                    MedicalRoleDataSchema, StrictPagination, None
+                ].model_validate(response.json())
+                data = DataPair[List[MedicalRoleDataSchema], None](
+                    old=validated_response.data,
+                    new=None,
+                )
+                result = ReadMultipleResourceOperationResult[
+                    MedicalRoleDataSchema, StrictPagination, None
+                ](
+                    data=data,
+                    pagination=validated_response.pagination,
+                    metadata=None,
+                    other=None,
+                )
+                ReadMultipleResourceOperationSchema[
+                    MedicalRoleDataSchema, StrictPagination, None
+                ](
+                    service_context=self._service_context,
+                    id=operation_id,
+                    context=operation_context,
+                    timestamp=OperationTimestamp(
+                        executed_at=executed_at,
+                        completed_at=completed_at,
+                        duration=(completed_at - executed_at).total_seconds(),
+                    ),
+                    summary="Successfully retrieved multiple medical role specializations",
+                    request_context=request_context,
+                    authentication=authentication,
+                    action=operation_action,
+                    resource=self._resource,
+                    result=result,
+                ).log(
+                    self._logger, level=LogLevel.INFO
+                )
+                await self._redis.set(
+                    cache_key, result.model_dump_json(), Expiration.EXP_1WK
+                )
+                return result
+
+            self._raise_resource_http_request_error(
+                response=response,
+                operation_id=operation_id,
+                operation_context=operation_context,
+                executed_at=executed_at,
+                operation_action=operation_action,
+                request_context=request_context,
+                authentication=authentication,
+                resource=self._resource,
+            )
+        else:
+            completed_at = datetime.now(tz=timezone.utc)
+            error = MaleoNotImplemented(
+                service_context=self._service_context,
+                operation_id=operation_id,
+                operation_context=operation_context,
+                operation_timestamp=OperationTimestamp(
+                    executed_at=executed_at,
+                    completed_at=completed_at,
+                    duration=(completed_at - executed_at).total_seconds(),
+                ),
+                operation_summary=f"The provided controller type '{controller_type}' did not exists",
+                operation_action=operation_action,
+                request_context=request_context,
+                authentication=authentication,
+                resource=self._resource,
+            )
+            error.operation_schema.log(self._logger, LogLevel.CRITICAL)
+            raise error
