@@ -6,7 +6,7 @@ from maleo_metadata.types.base.gender import IdentifierValueType
 
 
 IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
-    IdentifierType, Callable[[str], IdentifierValueType]
+    IdentifierType, Callable[..., IdentifierValueType]
 ] = {
     IdentifierType.ID: int,
     IdentifierType.UUID: UUID,
@@ -16,6 +16,6 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 
 
 RESOURCE = Resource(
-    identifier=ResourceIdentifier(key="genders", name="Genders", url_slug="genders"),
+    identifiers=[ResourceIdentifier(key="genders", name="Genders", url_slug="genders")],
     details=None,
 )
