@@ -7,7 +7,7 @@ from maleo_metadata.types.base.medical_role import IdentifierValueType
 
 IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
     IdentifierType,
-    Callable[[str], IdentifierValueType],
+    Callable[..., IdentifierValueType],
 ] = {
     IdentifierType.ID: int,
     IdentifierType.UUID: UUID,
@@ -18,8 +18,10 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 
 
 RESOURCE = Resource(
-    identifier=ResourceIdentifier(
-        key="medical_roles", name="MedicalRoled", url_slug="medical-roles"
-    ),
+    identifiers=[
+        ResourceIdentifier(
+            key="medical_roles", name="MedicalRoled", url_slug="medical-roles"
+        )
+    ],
     details=None,
 )
