@@ -6,7 +6,7 @@ from maleo_metadata.types.base.service import IdentifierValueType
 
 
 IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
-    IdentifierType, Callable[[str], IdentifierValueType]
+    IdentifierType, Callable[..., IdentifierValueType]
 ] = {
     IdentifierType.ID: int,
     IdentifierType.UUID: UUID,
@@ -16,6 +16,8 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 
 
 RESOURCE = Resource(
-    identifier=ResourceIdentifier(key="services", name="Services", url_slug="services"),
+    identifiers=[
+        ResourceIdentifier(key="services", name="Services", url_slug="services")
+    ],
     details=None,
 )
