@@ -7,7 +7,7 @@ from maleo_metadata.types.base.system_role import IdentifierValueType
 
 IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
     IdentifierType,
-    Callable[[str], IdentifierValueType],
+    Callable[..., IdentifierValueType],
 ] = {
     IdentifierType.ID: int,
     IdentifierType.UUID: UUID,
@@ -17,8 +17,10 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 
 
 RESOURCE = Resource(
-    identifier=ResourceIdentifier(
-        key="system_roles", name="SystemRoles", url_slug="system-roles"
-    ),
+    identifiers=[
+        ResourceIdentifier(
+            key="system_roles", name="SystemRoles", url_slug="system-roles"
+        )
+    ],
     details=None,
 )
