@@ -7,7 +7,7 @@ from maleo_metadata.types.base.user_type import IdentifierValueType
 
 IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
     IdentifierType,
-    Callable[[str], IdentifierValueType],
+    Callable[..., IdentifierValueType],
 ] = {
     IdentifierType.ID: int,
     IdentifierType.UUID: UUID,
@@ -17,8 +17,8 @@ IDENTIFIER_TYPE_VALUE_TYPE_MAP: Dict[
 
 
 RESOURCE = Resource(
-    identifier=ResourceIdentifier(
-        key="user_types", name="UserTypes", url_slug="user-types"
-    ),
+    identifiers=[
+        ResourceIdentifier(key="user_types", name="UserTypes", url_slug="user-types")
+    ],
     details=None,
 )
